(* bsg -- solve the balanced subgraph problem
   Copyright (C) 2006  Falk Hffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  *)

open Bsg;;

let gadgets = [|
  (* 0 *)
  [];

  (* 1 *)
  [];
  
  (* 2 *)
  [
    (0,[|0;1|],[(0,1,{eq=1;ne=0});]);
    (0,[|1;0|],[(0,1,{eq=0;ne=1});]);
  ];
  
  (* 3 *)
  [
    (0,[|0;0;1;1|],[(1,2,{eq=1;ne=0});]);
    (0,[|0;1;0;1|],[(0,2,{eq=1;ne=0});]);
    (0,[|0;1;1;0|],[(0,1,{eq=1;ne=0});]);
    (0,[|1;0;0;1|],[(0,1,{eq=0;ne=1});]);
    (0,[|1;0;1;0|],[(0,2,{eq=0;ne=1});]);
    (0,[|1;1;0;0|],[(1,2,{eq=0;ne=1});]);
    (1,[|0;1;1;1|],[(2,3,{eq=0;ne=1});(1,3,{eq=0;ne=1});(0,3,{eq=0;ne=1});]);
    (1,[|1;0;1;1|],[(2,3,{eq=1;ne=0});(1,3,{eq=1;ne=0});(0,3,{eq=0;ne=1});]);
    (1,[|1;1;0;1|],[(2,3,{eq=0;ne=1});(1,3,{eq=1;ne=0});(0,3,{eq=0;ne=1});]);
    (1,[|1;1;1;0|],[(2,3,{eq=1;ne=0});(1,3,{eq=0;ne=1});(0,3,{eq=0;ne=1});]);
  ];

  (* 4 *)
  [
    (0,[|0;0;0;0;1;1;1;1|],[(2,3,{eq=1;ne=0});]);
    (0,[|0;0;1;1;0;0;1;1|],[(1,3,{eq=1;ne=0});]);
    (0,[|0;0;1;1;1;1;0;0|],[(1,2,{eq=1;ne=0});]);
    (0,[|0;1;0;1;0;1;0;1|],[(0,3,{eq=1;ne=0});]);
    (0,[|0;1;0;1;1;0;1;0|],[(0,2,{eq=1;ne=0});]);
    (0,[|0;1;1;0;0;1;1;0|],[(0,1,{eq=1;ne=0});]);
    (0,[|1;0;0;1;1;0;0;1|],[(0,1,{eq=0;ne=1});]);
    (0,[|1;0;1;0;0;1;0;1|],[(0,2,{eq=0;ne=1});]);
    (0,[|1;0;1;0;1;0;1;0|],[(0,3,{eq=0;ne=1});]);
    (0,[|1;1;0;0;0;0;1;1|],[(1,2,{eq=0;ne=1});]);
    (0,[|1;1;0;0;1;1;0;0|],[(1,3,{eq=0;ne=1});]);
    (0,[|1;1;1;1;0;0;0;0|],[(2,3,{eq=0;ne=1});]);
    (1,[|0;0;1;1;1;1;1;1|],[(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});]);
    (1,[|0;1;0;1;1;1;1;1|],[(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|0;1;1;1;0;1;1;1|],[(3,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|0;1;1;1;1;1;1;0|],[(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|0;1;1;2;1;2;2;1|],[(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|1;0;1;0;1;1;1;1|],[(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;0;1;1;1;0;1;1|],[(3,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;0;1;1;1;1;0;1|],[(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;0;2;1;2;1;1;2|],[(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;0;0;1;1;1;1|],[(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});]);
    (1,[|1;1;0;1;1;0;1;1|],[(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;0;1;1;1;0;1|],[(3,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;1;0;0;1;1;1|],[(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;1;0;1;1;1;0|],[(3,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;1;1;0;0;1;1|],[(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});]);
    (1,[|1;1;1;1;0;1;0;1|],[(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;1;1;1;0;1;0|],[(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|1;1;1;1;1;1;0;0|],[(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});]);
    (1,[|1;2;0;1;2;1;1;2|],[(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|1;2;2;1;0;1;1;2|],[(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|1;2;2;1;2;1;1;0|],[(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|2;1;1;0;1;2;2;1|],[(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (1,[|2;1;1;2;1;0;2;1|],[(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (1,[|2;1;1;2;1;2;0;1|],[(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|0;1;1;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|0;1;1;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|0;1;1;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|0;1;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|0;1;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|0;1;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|0;2;1;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;0;1;1;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;0;2;1;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;0;2;1;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;0;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;0;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;0;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;0;1;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;0;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;1;0;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;1;2;0;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;0;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;1;1;0|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;1;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;0;1;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;0;1;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;0;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;0;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;0;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;0;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;1;1;0|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;1;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;1;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;0;1;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;0;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;0;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;1;1;0|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;1;2;0|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;1;0|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;0;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;1;0|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;3;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;3;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;3;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;2;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;1;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;2;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;3;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;4;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|1;3;3;4;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;4;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|1;3;3;4;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;0;2;1;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;0;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;0;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;0;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;0;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;1;0;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;1;2;0;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;1;0;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;1;0;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;1;1;0;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;1;2;0;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;0;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;0;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;0;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;0;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;0;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;2;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;1;3;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;0;1;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;0;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;1;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;0;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;0;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;2;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;1;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;1;3;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;0;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;1;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;1;0|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;2;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;1;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;2;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;1;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;1;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;2;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;2;3;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;3;4;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;2;4;3;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;1;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;1;3;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;1;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;2;4;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;2;2;4;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;3;3;3;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;2;3;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|2;4;4;2;2;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;2;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;3;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;4;3;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;1;4;3;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;4;3;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;1;4;3;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;1;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;1;2;2;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;1;1;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;1;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;1;2;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;1;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;1;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;2;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;2;3;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;1;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;1;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;2;3;4;2;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;3;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;2;3;2;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;2;4;2;4;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;2;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;1;3;3;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;1;3;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;2;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;2;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;2;3;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;2;3;3;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;2;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;1;3;3;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;1;3;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;2;3;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;2;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;1;3;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;1;3;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;2;3;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;1;3;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;2;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;1;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;3;3;3;3;3;2;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});]);
    (2,[|3;4;1;3;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;1;3;4;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;1;3;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;1;3;4;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;2;2;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;2;2;4;3;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;1;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;1;3;3;4|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;1;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;1;3;3;4|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;4;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|3;4;4;3;4;3;3;1|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;4;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|3;4;4;3;4;3;3;1|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;2;4;3;2;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;2;4;2;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;2;3;2;3;2;4;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;2;2;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;2;2;3;4;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;1;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;1;3;4;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;1;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;1;3;4;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;2;2;2;2|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;3;1;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;3;1;4;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;3;1;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;3;1;4;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;3;4;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=0;ne=1});(3,4,{eq=0;ne=1});(2,5,{eq=1;ne=0});(2,4,{eq=1;ne=0});(1,5,{eq=1;ne=0});(1,4,{eq=1;ne=0});(0,5,{eq=0;ne=1});(0,4,{eq=0;ne=1});]);
    (2,[|4;3;3;4;3;4;1;3|],[(4,5,{eq=0;ne=1});(3,5,{eq=1;ne=0});(3,4,{eq=1;ne=0});(2,5,{eq=0;ne=1});(2,4,{eq=0;ne=1});(1,5,{eq=0;ne=1});(1,4,{eq=0;ne=1});(0,5,{eq=1;ne=0});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;3;4;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=0;ne=1});(3,4,{eq=1;ne=0});(2,5,{eq=1;ne=0});(2,4,{eq=0;ne=1});(1,5,{eq=1;ne=0});(1,4,{eq=0;ne=1});(0,5,{eq=0;ne=1});(0,4,{eq=1;ne=0});]);
    (2,[|4;3;3;4;3;4;1;3|],[(4,5,{eq=1;ne=0});(3,5,{eq=1;ne=0});(3,4,{eq=0;ne=1});(2,5,{eq=0;ne=1});(2,4,{eq=1;ne=0});(1,5,{eq=0;ne=1});(1,4,{eq=1;ne=0});(0,5,{eq=1;ne=0});(0,4,{eq=0;ne=1});]);
  ]
|];;
